#!/bin/bash

CLI_QDISC=""
CQM_QDISC="cake"
CDL_QDISC="netem delay 50ms"

SDL_QDISC="netem delay 50ms"
SQM_QDISC="cake"
SRV_QDISC=""

# namespaces:
# cli- client
# cqm- client queue manager
# cdl- client delay
# sdl- server delay
# sqm- server queue manager
# srv- server
#
# topology:
# cli <-> cqm <-> cdl <-> sdl <-> sqm <-> srv

# sets up namespace
# <mode:right/bridge/left> <name> <desc> <params>
# right params: <ip> <left ns>
# bridge params: <left ns>
# left params: <ip>
ns() {
	local mode="$1"; shift
	local name="$1"; shift
	local desc="$1"; shift

	echo -e "\n----- $mode $name ($desc):\n"

	ip netns add $name

	case $mode in
	right)
		local ip="$1"; shift
		local leftns="$1"; shift
		ip link add dev ${name}.l type veth peer name ${leftns}.r
		ip link set dev ${name}.l netns $name
		ip netns exec ${name} ip addr add $ip dev ${name}.l
		ip netns exec ${name} ip link set ${name}.l up
		;;
	bridge)
		local leftns="$1"; shift
		ip link set dev ${name}.r netns $name
		ip netns exec $name ip link set ${name}.r up
		ip link add dev ${name}.l type veth peer name ${leftns}.r
		ip link set dev ${name}.l netns $name
		ip netns exec $name ip link set ${name}.l up
		ip netns exec $name ip link add name ${name}.b type bridge
		ip netns exec $name ip link set dev ${name}.r master ${name}.b
		ip netns exec $name ip link set dev ${name}.l master ${name}.b
		ip netns exec $name ip link set dev ${name}.b up
		;;
	left)
		local ip="$1"; shift
		ip link set dev ${name}.r netns $name
		ip netns exec $name ip addr add $ip dev ${name}.r
		ip netns exec $name ip link set ${name}.r up
		;;
	*)
		echo "invalid namespace mode: $mode"
		exit 1
		;;
	esac

	ip netns exec ${name} ip addr
}

on() {
	# exit on error
	set -e

	# set up server
	ns right srv "server" 10.12.0.1/24 sqm
	
	# set up sqm
	ns bridge sqm "server queue manager" sdl

	# set up sdl
	ns bridge sdl "server delay" cdl

	# set up cdl
	ns bridge cdl "client delay" cqm
	
	# set up cqm
	ns bridge cqm "client queue manager" cli
	
	# set up client
	ns left cli "client" 10.12.0.2/24

	# add netem qdiscs
	if [ "$CLI_QDISC" != "" ]; then
		ip netns exec cli tc qdisc add dev cli.r root $CLI_QDISC
	fi
	if [ "$CQM_QDISC" != "" ]; then
		ip netns exec cqm tc qdisc add dev cqm.r root $CQM_QDISC
	fi
	if [ "$CDL_DELAY" != "" ]; then
		ip netns exec cdl tc qdisc add dev cdl.r root $CDL_DELAY
	fi
	if [ "$SDL_QDISC" != "" ]; then
		ip netns exec sdl tc qdisc add dev sdl.l root $SDL_QDISC
	fi
	if [ "$SQM_QDISC" != "" ]; then
		ip netns exec sqm tc qdisc add dev sqm.l root $SQM_QDISC
	fi
	if [ "$SRV_QDISC" != "" ]; then
		ip netns exec srv tc qdisc add dev srv.l root $SRV_QDISC
	fi

	echo
	ip netns exec cli ping -c 3 -i 0.2 10.12.0.1
}

off() {
	ip netns del cli
	ip netns del cqm
	ip netns del cdl
	ip netns del sdl
	ip netns del sqm
	ip netns del srv
	echo "Namespaces deleted."
}

restart() {
	off
	on
}

eval "$1"
